﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using SWConfigDataClientLib;
using SWConfigDataClientLib.Proxies.Admin;
using SWConfigDataClientLib.WSUserPhoneBook;
using System.IO;
using SWConfigDataClientLib.Proxies.UserPhoneBook;
using IpPbx.Samples.Shared;

namespace IpPbx.EditPersonalPhonebook
{
    public partial class frmMain : Form
    {
        #region Fields & CTor

        private CDSLib cdsLib;
        private UserAdminView1EntrySortableCollection userView;
        private List<UserPhoneBookEntry> newPersonalPhonebookData;

        public frmMain()
        {
            InitializeComponent();
        }

        #endregion

        #region UI Interaction

        private void btnLogin_Click(object sender, EventArgs e)
        {
            this.cdsLib = new CDSLib(txtName.Text,
                                    txtPassword.Text,
                                    txtServer.Text);

            if (!this.cdsLib.TestConnection())
            {
                MessageBox.Show("Connection failed", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            if (!LoadUsers())
            {
                return;
            }

            txtName.Enabled = false;
            txtPassword.Enabled = false;
            txtServer.Enabled = false;
            btnLogin.Enabled = false;
            btnRefresh.Enabled = true;
            btnBrowse.Enabled = true;
            btnClearPhonebook.Enabled = true;            
        }

        //------------------------------------------------------------------------------------

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            LoadUsers();
        }

        //------------------------------------------------------------------------------------        

        private void cbxAllUsers_CheckedChanged(object sender, EventArgs e)
        {
            for (int x = 0; x < lbxUsers.Items.Count; x++)
            {
                lbxUsers.SetSelected(x, cbxAllUsers.Checked);
            }
        }

        //------------------------------------------------------------------------------------        

        private void btnBrowse_Click(object sender, EventArgs e)
        {
            var result = ofdCsv.ShowDialog();
            if (result != DialogResult.OK)
                return;

            try
            {
                newPersonalPhonebookData = LoadCsv(ofdCsv.FileName, true);
                
                if (newPersonalPhonebookData == null ||
                    newPersonalPhonebookData.Count == 0)
                {
                    throw new ArgumentException("File contains no valid data.");
                }

                txtCsvFilePath.Text = ofdCsv.FileName;
                cbxClearPersonalPhonebook.Enabled = true;
            }
            catch (Exception exception)
            {
                MessageBox.Show(exception.Message, "Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //------------------------------------------------------------------------------------

        private void btnImport_Click(object sender, EventArgs e)
        {
            try
            {
                var selectedUsers = lbxUsers.SelectedItems;

                if (selectedUsers == null ||
                    selectedUsers.Count == 0)
                {
                    throw new ArgumentException("No users selected.");
                }

                foreach (var user in selectedUsers)
                {
                    var userEntry = userView.FirstOrDefault(x => x.Name == user.ToString());
                    if (userEntry == null)
                        continue;

                    cdsLib.ImportPersonalPhonebook(userEntry,
                        newPersonalPhonebookData,
                        cbxClearPersonalPhonebook.Checked);
                }

                MessageBox.Show("Import completed successfully.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception exception)
            {
                MessageBox.Show(exception.Message, "Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //------------------------------------------------------------------------------------

        private void btnClearPhonebook_Click(object sender, EventArgs e)
        {
            try
            {
                var selectedUsers = lbxUsers.SelectedItems;

                if (selectedUsers == null ||
                    selectedUsers.Count == 0)
                {
                    throw new ArgumentException("No users selected.");
                }

                foreach (var user in selectedUsers)
                {
                    var userEntry = userView.FirstOrDefault(x => x.Name == user.ToString());
                    if (userEntry == null)
                        continue;

                    cdsLib.ClearPersonalPhonebook(userEntry);
                }

                MessageBox.Show("Process completed successfully.", "Information", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (Exception exception)
            {
                MessageBox.Show(exception.Message, "Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        //------------------------------------------------------------------------------------

        #endregion UI Interaction

        #region Private Methods
               
        private bool LoadUsers()
        {
            try
            {
                lbxUsers.Items.Clear();

                userView = cdsLib.GetUserView(true);
                
                foreach (var entry in userView)
                {
                    // Filter builtin user (like Conference & MEM)
                    if (!entry.IsShortnameNull)
                        continue;

                    lbxUsers.Items.Add(entry.Name);                    
                }

                lbxUsers.Enabled = true;
                cbxAllUsers.Enabled = true;
                btnImport.Enabled = true;
                
                return true;
            }
            catch (Exception exception)
            {
                MessageBox.Show(exception.Message, "Exception", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            return false;
        }

        //------------------------------------------------------------------------------------        

        private List<UserPhoneBookEntry> LoadCsv(string filename, bool ignoreHeader)
        {
            var lines = File.ReadAllLines(filename, Encoding.UTF8);

            if (ignoreHeader)
            {
                lines = lines.Skip(1).ToArray();  
            }

            var num_rows = lines.Length;

            var values = new List<UserPhoneBookEntry>();

            for (var r = 0; r < num_rows; r++)
            {
                var row = lines[r].Split(';');

                var entry = new UserPhoneBookEntry();                
                entry.Name = string.Format("{0} {1}", row[0], row[1]);
                entry.Number = row[2];
                entry.SearchNumber = row[2];

                bool isHidden;
                if (Boolean.TryParse(row[3], out isHidden))
                {
                    entry.Hide = isHidden;
                }

                values.Add(entry);
            }

            return values;
        }

        #endregion

    }
}
